export type FSMLifecycleFn<StatesT extends string, EventsT extends string> = (meta: LifecycleFnMeta<StatesT, EventsT>) => void;
export type LifecycleFnMeta<StatesT extends string, EventsT extends string> = {
    from: StatesT | null;
    to: StatesT;
    event: EventsT | null;
    args: any;
};
export declare function isLifecycleFnMeta(meta: unknown): meta is LifecycleFnMeta<any, any>;
export type FSMLifecycle = '_enter' | '_exit';
export type Action<StatesT> = StatesT | ((...args: any[]) => StatesT | void);
export type StateHandler<StatesT extends string, EventsT extends string> = {
    [e in EventsT]?: Action<StatesT>;
} & {
    [k in FSMLifecycle]?: FSMLifecycleFn<StatesT, EventsT>;
};
export type Transition<StatesT extends string, EventsT extends string> = {
    [s in StatesT]: StateHandler<StatesT, EventsT>;
} & {
    '*'?: StateHandler<StatesT, EventsT>;
};
export declare function fsm<StatesT extends string, EventsT extends string>(initial: StatesT, states: Transition<StatesT, EventsT>): {
    readonly current: StatesT;
    send(event: EventsT, ...args: any[]): StatesT;
    debounce(wait: number | undefined, event: EventsT, ...args: any[]): Promise<StatesT>;
};
