export function isLifecycleFnMeta(meta) {
    return (!!meta &&
        typeof meta === 'object' &&
        'to' in meta &&
        'from' in meta &&
        'event' in meta &&
        'args' in meta);
}
export function fsm(initial, states) {
    let current = $state(initial);
    const transition = (newState, event, args) => {
        const metadata = { from: current, to: newState, event, args };
        dispatch('_exit', metadata);
        current = newState;
        dispatch('_enter', metadata);
    };
    const dispatch = (event, ...args) => {
        const action = states[current]?.[event] ?? states['*']?.[event];
        if (action instanceof Function) {
            if (event === '_enter' || event === '_exit') {
                if (isLifecycleFnMeta(args[0])) {
                    action(args[0]);
                    return;
                }
                else {
                    console.warn('Invalid metadata passed to lifecycle function of the FSM.');
                }
            }
            else {
                return action(...args);
            }
        }
        else if (typeof action === 'string') {
            return action;
        }
        else {
            if (event !== '_enter' && event !== '_exit') {
                console.warn('No action defined for event', event, 'in state', current);
            }
        }
    };
    const timeout = {};
    // synthetically trigger _enter for the initial state.
    dispatch('_enter', { from: null, to: initial, event: null, args: [] });
    return {
        get current() {
            return current;
        },
        send(event, ...args) {
            const newState = dispatch(event, ...args);
            if (newState && newState !== current) {
                transition(newState, event, args);
            }
            return current;
        },
        async debounce(wait = 500, event, ...args) {
            clearTimeout(timeout[event]);
            await new Promise((resolve) => (timeout[event] = setTimeout(resolve, wait)));
            delete timeout[event];
            return new Promise((resolve) => resolve(this.send(event, ...args)));
        }
    };
}
